// Read a file of first/last names and scores on components.  Keep
// track of a set of aggregate statistics like mean, max, and total
// for each column.
// 
// Handles any number of columns by counting the number of words in
// the first line which is a header
//
// Sample scores-all.dat file
//
// First  Last      HW1   HW2  Exam1
// Lee    Adama    17.0  12.0   34.0
// Gaius  Baltar   18.0  13.0   38.0
// Kara   Thrace   16.0  10.0   24.5
// Saul   Tye      10.0  13.5   34.0
// Laura  Rosslyn  15.0  12.0   36.0


import java.io.*;
import java.util.Scanner;
public class ComplexStats{

  // Controls which statistics are computed
  public static Statistic makeStat(){
    // Statistic s = new Mean();
    // Statistic s = new Max();
    // Statistic s = new Total();
    Statistic s = new AggregateStat(new Statistic[]{
    	new Mean(), 
    	new Total(),
    	new Max(),
      });
    return s;
  }

  // Count the words in a string
  public static int countWords(String s){
    Scanner sin = new Scanner(s);
    int n = 0;
    while(sin.hasNext()){
      sin.next();
      n++;
    }
    sin.close();
    return n;
  }
  
  public static void main(String args[])
    throws Exception // more specifically FileNotFoundException 
  {
    String fname = "scores-all.dat";
    Scanner in = new Scanner(new File(fname));
    // Read the header
    String header = in.nextLine(); // Read whole line
    int ncols = countWords(header);
    System.out.printf("%d columns\n",ncols);
    int nComp = ncols-2;	// Assumes first/last name
    String compNames[] = new String[nComp];
    Statistic stats[] = new Statistic[nComp];
    Scanner sin = new Scanner(header);
    sin.next(); sin.next();	// Skip first/last

    // Set up the Statistics and component names
    for(int i=0; i<nComp; i++){
      compNames[i] = sin.next(); // Read from header, not file
      stats[i] = makeStat();
    }
    sin.close();

    // Read until end of file
    while(in.hasNext()){
      String name = in.next() + " " + in.next();
      for(int i=0; i<nComp; i++){
	double score = in.nextDouble();
	stats[i].update(name,score);
      }
    }
    in.close();
    for(int i=0; i<nComp;i++){
      System.out.printf("%8s %s\n", compNames[i], stats[i].toString());
    }
  }
  

}
